@@ -2,7 +2,7 @@ package Text::PDF::Name;
 
 use strict;
 use vars qw(@ISA);
-no warnings qw(uninitialized);
+# no warnings qw(uninitialized);
 
 use Text::PDF::String;
 @ISA = qw(Text::PDF::String);
